aboutsummaryrefslogtreecommitdiffstats
path: root/src/java/com/sun/gluegen
diff options
context:
space:
mode:
Diffstat (limited to 'src/java/com/sun/gluegen')
-rw-r--r--src/java/com/sun/gluegen/CMethodBindingEmitter.java7
-rw-r--r--src/java/com/sun/gluegen/JavaConfiguration.java4
-rw-r--r--src/java/com/sun/gluegen/JavaEmitter.java86
-rw-r--r--src/java/com/sun/gluegen/JavaMethodBindingEmitter.java14
-rw-r--r--src/java/com/sun/gluegen/JavaType.java14
-rw-r--r--src/java/com/sun/gluegen/StructLayout.java2
-rw-r--r--src/java/com/sun/gluegen/test/TestPointerBufferEndian.java4
-rw-r--r--src/java/com/sun/gluegen/test/TestStructAccessorEndian.java4
8 files changed, 102 insertions, 33 deletions
diff --git a/src/java/com/sun/gluegen/CMethodBindingEmitter.java b/src/java/com/sun/gluegen/CMethodBindingEmitter.java
index c91e992..022df39 100644
--- a/src/java/com/sun/gluegen/CMethodBindingEmitter.java
+++ b/src/java/com/sun/gluegen/CMethodBindingEmitter.java
@@ -1009,7 +1009,12 @@ public class CMethodBindingEmitter extends FunctionEmitter
System.err.println(
"WARNING: No capacity specified for java.nio.Buffer return " +
"value for function \"" + binding + "\";" +
- " assuming size of equivalent C return type (sizeof(" + cReturnType.getName() + ")): " + binding);
+ " assuming size of equivalent C return type (sizeof(" + cReturnType.getName() + ")): " + binding);
+ /**
+ throw new RuntimeException(
+ "WARNING: No capacity specified for java.nio.Buffer return " +
+ "value for function \"" + binding + "\";" +
+ " C return type is " + cReturnType.getName() + ": " + binding); */
}
writer.println(");");
} else if (javaReturnType.isString()) {
diff --git a/src/java/com/sun/gluegen/JavaEmitter.java b/src/java/com/sun/gluegen/JavaEmitter.java
index 0759d81..25aeb16 100644
--- a/src/java/com/sun/gluegen/JavaEmitter.java
+++ b/src/java/com/sun/gluegen/JavaEmitter.java
@@ -700,7 +700,7 @@ public class JavaEmitter implements GlueEmitter {
// arg_direct,
// ... );
// }
- // private native void fooMethod0(Object arg, int arg_byte_offset, boolean arg_is_direct, ...);
+ // private native void fooMethod1(Object arg, int arg_byte_offset, boolean arg_is_direct, ...);
//
// Method taking primitive array argument:
// Interface class:
@@ -713,13 +713,15 @@ public class JavaEmitter implements GlueEmitter {
// }
// public void fooMethod(IntBuffer arg) {
// ... bounds checks, etc. ...
- // if (arg.isDirect()) {
- // fooMethod0(arg, computeDirectBufferByteOffset(arg));
- // } else {
- // fooMethod1(getIndirectBufferArray(arg), computeIndirectBufferByteOffset(arg));
- // }
+ //
+ // boolean arg_direct = BufferFactory.isDirect(arg);
+ //
+ // fooMethod1(arg_direct?arg:BufferFactory.getArray(arg),
+ // arg_direct?BufferFactory.getDirectBufferByteOffset(arg):BufferFactory.getIndirectBufferByteOffset(arg),
+ // arg_direct,
+ // ... );
// }
- // private native void fooMethod0(Object arg, int arg_byte_offset, boolean arg_is_direct, ...);
+ // private native void fooMethod1(Object arg, int arg_byte_offset, boolean arg_is_direct, ...);
//
// Note in particular that the public entry point taking an
// array is merely a special case of the indirect buffer case.
@@ -909,7 +911,7 @@ public class JavaEmitter implements GlueEmitter {
}
writer.println(" public static int size() {");
if (doBaseClass) {
- writer.println(" if (CPU.is32Bit()) {");
+ writer.println(" if (Platform.is32Bit()) {");
writer.println(" return " + containingTypeName + "32" + ".size();");
writer.println(" } else {");
writer.println(" return " + containingTypeName + "64" + ".size();");
@@ -925,7 +927,7 @@ public class JavaEmitter implements GlueEmitter {
writer.println(" }");
writer.println();
writer.println(" public static " + containingTypeName + " create(java.nio.ByteBuffer buf) {");
- writer.println(" if (CPU.is32Bit()) {");
+ writer.println(" if (Platform.is32Bit()) {");
writer.println(" return new " + containingTypeName + "32(buf);");
writer.println(" } else {");
writer.println(" return new " + containingTypeName + "64(buf);");
@@ -1213,13 +1215,40 @@ public class JavaEmitter implements GlueEmitter {
(opt.getTargetType().getName().equals("JNIEnv"))) {
return JavaType.createForJNIEnv();
}
+ Type t = cType;
// Opaque specifications override automatic conversions
- TypeInfo info = cfg.typeInfo(cType, typedefDictionary);
+ // in case the identity is being used .. not if ptr-ptr
+ TypeInfo info = cfg.typeInfo(t, typedefDictionary);
if (info != null) {
- return info.javaType();
+ boolean isPointerPointer = false;
+ if (t.pointerDepth() > 0 || t.arrayDimension() > 0) {
+ Type targetType; // target type
+ if (t.isPointer()) {
+ // t is <type>*, we need to get <type>
+ targetType = t.asPointer().getTargetType();
+ } else {
+ // t is <type>[], we need to get <type>
+ targetType = t.asArray().getElementType();
+ }
+ if (t.pointerDepth() == 2 || t.arrayDimension() == 2) {
+ // Get the target type of the target type (targetType was computer earlier
+ // as to be a pointer to the target type, so now we need to get its
+ // target type)
+ if (targetType.isPointer()) {
+ isPointerPointer = true;
+
+ // t is<type>**, targetType is <type>*, we need to get <type>
+ Type bottomType = targetType.asPointer().getTargetType();
+ System.out.println("INFO: Opaque Type: "+t+", targetType: "+targetType+", bottomType: "+bottomType+" is ptr-ptr");
+ }
+ }
+ }
+ if(!isPointerPointer) {
+ return info.javaType();
+ }
}
- Type t = cType;
+
if (t.isInt() || t.isEnum()) {
switch ((int) t.getSize(curMachDesc)) {
case 1: return javaType(Byte.TYPE);
@@ -1300,11 +1329,16 @@ public class JavaEmitter implements GlueEmitter {
if (targetType.isPointer()) {
// t is<type>**, targetType is <type>*, we need to get <type>
bottomType = targetType.asPointer().getTargetType();
+ return JavaType.forNIOPointerBufferClass();
} else {
// t is<type>[][], targetType is <type>[], we need to get <type>
bottomType = targetType.asArray().getElementType();
+ System.out.println("WARNING: typeToJavaType(ptr-ptr): "+t+", targetType: "+targetType+", bottomType: "+bottomType+" -> Unhandled!");
}
+ // Warning: The below code is not backed up by an implementation,
+ // the only working variant is a ptr-ptr type which results in a PointerBuffer.
+ //
if (bottomType.isPrimitive()) {
if (bottomType.isInt()) {
switch ((int) bottomType.getSize(curMachDesc)) {
@@ -1698,6 +1732,8 @@ public class JavaEmitter implements GlueEmitter {
binding.renameMethodName(cfg.getJavaSymbolRename(sym.getName()));
+ // System.out.println("bindFunction(0) "+sym.getReturnType());
+
if (cfg.returnsString(binding.getName())) {
PointerType prt = sym.getReturnType().asPointer();
if (prt == null ||
@@ -1712,6 +1748,8 @@ public class JavaEmitter implements GlueEmitter {
binding.setJavaReturnType(typeToJavaType(sym.getReturnType(), false, curMachDesc));
}
+ // System.out.println("bindFunction(1) "+binding.getJavaReturnType());
+
// List of the indices of the arguments in this function that should be
// converted from byte[] or short[] to String
List<Integer> stringArgIndices = cfg.stringArguments(binding.getName());
@@ -1719,22 +1757,23 @@ public class JavaEmitter implements GlueEmitter {
for (int i = 0; i < sym.getNumArguments(); i++) {
Type cArgType = sym.getArgumentType(i);
JavaType mappedType = typeToJavaType(cArgType, true, curMachDesc);
- //System.out.println("C arg type -> \"" + cArgType + "\"" );
- //System.out.println(" Java -> \"" + mappedType + "\"" );
+ // System.out.println("C arg type -> \"" + cArgType + "\"" );
+ // System.out.println(" Java -> \"" + mappedType + "\"" );
// Take into account any ArgumentIsString configuration directives that apply
if (stringArgIndices != null && stringArgIndices.contains(i)) {
- //System.out.println("Forcing conversion of " + binding.getName() + " arg #" + i + " from byte[] to String ");
+ // System.out.println("Forcing conversion of " + binding.getName() + " arg #" + i + " from byte[] to String ");
if (mappedType.isCVoidPointerType() ||
mappedType.isCCharPointerType() ||
mappedType.isCShortPointerType() ||
+ mappedType.isNIOPointerBuffer() ||
(mappedType.isArray() &&
(mappedType.getJavaClass() == ArrayTypes.byteBufferArrayClass) ||
(mappedType.getJavaClass() == ArrayTypes.shortBufferArrayClass))) {
// convert mapped type from:
// void*, byte[], and short[] to String
// ByteBuffer[] and ShortBuffer[] to String[]
- if (mappedType.isArray()) {
+ if (mappedType.isArray() || mappedType.isNIOPointerBuffer()) {
mappedType = javaType(ArrayTypes.stringArrayClass);
} else {
mappedType = javaType(String.class);
@@ -1751,8 +1790,9 @@ public class JavaEmitter implements GlueEmitter {
//System.out.println("During binding of [" + sym + "], added mapping from C type: " + cArgType + " to Java type: " + mappedType);
}
- //System.err.println("---> " + binding);
- //System.err.println(" ---> " + binding.getCSymbol());
+ // System.out.println("---> " + binding);
+ // System.out.println(" ---> " + binding.getCSymbol());
+ // System.out.println("bindFunction(3) "+binding);
return binding;
}
@@ -1762,6 +1802,8 @@ public class JavaEmitter implements GlueEmitter {
MethodBinding result = inputBinding;
boolean arrayPossible = false;
+ // System.out.println("lowerMethodBindingPointerTypes(0): "+result);
+
for (int i = 0; i < inputBinding.getNumArguments(); i++) {
JavaType t = inputBinding.getJavaArgumentType(i);
if (t.isCPrimitivePointerType()) {
@@ -1794,7 +1836,7 @@ public class JavaEmitter implements GlueEmitter {
if (convertToArrays) {
result = result.replaceJavaArgumentType(i, javaType(ArrayTypes.longArrayClass));
} else {
- result = result.replaceJavaArgumentType(i, JavaType.forNIOPointerBufferClass());
+ result = result.replaceJavaArgumentType(i, JavaType.forNIOInt64BufferClass());
}
} else if (t.isCFloatPointerType()) {
arrayPossible = true;
@@ -1816,6 +1858,8 @@ public class JavaEmitter implements GlueEmitter {
}
}
+ // System.out.println("lowerMethodBindingPointerTypes(1): "+result);
+
// Always return primitive pointer types as NIO buffers
JavaType t = result.getJavaReturnType();
if (t.isCPrimitivePointerType()) {
@@ -1828,7 +1872,7 @@ public class JavaEmitter implements GlueEmitter {
} else if (t.isCInt32PointerType()) {
result = result.replaceJavaArgumentType(-1, JavaType.forNIOIntBufferClass());
} else if (t.isCInt64PointerType()) {
- result = result.replaceJavaArgumentType(-1, JavaType.forNIOPointerBufferClass());
+ result = result.replaceJavaArgumentType(-1, JavaType.forNIOInt64BufferClass());
} else if (t.isCFloatPointerType()) {
result = result.replaceJavaArgumentType(-1, JavaType.forNIOFloatBufferClass());
} else if (t.isCDoublePointerType()) {
@@ -1838,6 +1882,8 @@ public class JavaEmitter implements GlueEmitter {
}
}
+ // System.out.println("lowerMethodBindingPointerTypes(2): "+result);
+
if (canProduceArrayVariant != null) {
canProduceArrayVariant[0] = arrayPossible;
}
diff --git a/src/java/com/sun/gluegen/JavaMethodBindingEmitter.java b/src/java/com/sun/gluegen/JavaMethodBindingEmitter.java
index 8db4a3b..4153e32 100644
--- a/src/java/com/sun/gluegen/JavaMethodBindingEmitter.java
+++ b/src/java/com/sun/gluegen/JavaMethodBindingEmitter.java
@@ -589,7 +589,7 @@ public class JavaMethodBindingEmitter extends FunctionEmitter
}
if (type.isNIOBuffer()) {
- if(type.isNIOPointerBuffer()) {
+ if(type.isNIOInt64Buffer() || type.isNIOPointerBuffer()) {
if (directNIOOnly) {
writer.print( getArgumentName(i)+ " != null ? " + getArgumentName(i) + ".getBuffer() : null");
} else {
@@ -730,11 +730,17 @@ public class JavaMethodBindingEmitter extends FunctionEmitter
if (!returnType.isNIOByteBuffer()) {
// See whether we have to expand pointers to longs
if (getBinding().getCReturnType().pointerDepth() >= 2) {
- if (!returnType.isNIOPointerBuffer()) {
+ if (returnType.isNIOPointerBuffer()) {
+ writer.println(" return PointerBuffer.wrap(_res);");
+ } else if (returnType.isNIOInt64Buffer()) {
+ writer.println(" return Int64Buffer.wrap(_res);");
+ } else {
throw new RuntimeException("While emitting glue code for " + getName() +
- ": can not legally make pointers opaque to anything but longs");
+ ": can not legally make pointers opaque to anything but PointerBuffer or Int64Buffer/long");
}
- writer.println(" return PointerBuffer.wrap(_res);");
+ } else if (getBinding().getCReturnType().pointerDepth() == 1 &&
+ returnType.isNIOInt64Buffer()) {
+ writer.println(" return Int64Buffer.wrap(_res);");
} else {
String returnTypeName = returnType.getName().substring("java.nio.".length());
writer.println(" return _res.as" + returnTypeName + "();");
diff --git a/src/java/com/sun/gluegen/JavaType.java b/src/java/com/sun/gluegen/JavaType.java
index 19ac945..23f48b2 100644
--- a/src/java/com/sun/gluegen/JavaType.java
+++ b/src/java/com/sun/gluegen/JavaType.java
@@ -71,6 +71,7 @@ public class JavaType {
private static JavaType nioIntBufferType;
private static JavaType nioLongBufferType;
private static JavaType nioPointerBufferType;
+ private static JavaType nioInt64BufferType;
private static JavaType nioFloatBufferType;
private static JavaType nioDoubleBufferType;
private static JavaType nioByteBufferArrayType;
@@ -194,6 +195,12 @@ public class JavaType {
return nioLongBufferType;
}
+ public static JavaType forNIOInt64BufferClass() {
+ if(nioInt64BufferType == null)
+ nioInt64BufferType = createForClass(com.jogamp.gluegen.runtime.Int64Buffer.class);
+ return nioInt64BufferType;
+ }
+
public static JavaType forNIOPointerBufferClass() {
if(nioPointerBufferType == null)
nioPointerBufferType = createForClass(com.jogamp.gluegen.runtime.PointerBuffer.class);
@@ -333,7 +340,8 @@ public class JavaType {
public boolean isNIOBuffer() {
return clazz != null && ( (java.nio.Buffer.class).isAssignableFrom(clazz) ||
- (com.jogamp.gluegen.runtime.PointerBuffer.class).isAssignableFrom(clazz) ) ;
+ (com.jogamp.gluegen.runtime.PointerBuffer.class).isAssignableFrom(clazz) ||
+ (com.jogamp.gluegen.runtime.Int64Buffer.class).isAssignableFrom(clazz) ) ;
}
public boolean isNIOByteBuffer() {
@@ -353,6 +361,10 @@ public class JavaType {
return (clazz == java.nio.LongBuffer.class);
}
+ public boolean isNIOInt64Buffer() {
+ return (clazz == com.jogamp.gluegen.runtime.Int64Buffer.class);
+ }
+
public boolean isNIOPointerBuffer() {
return (clazz == com.jogamp.gluegen.runtime.PointerBuffer.class);
}
diff --git a/src/java/com/sun/gluegen/StructLayout.java b/src/java/com/sun/gluegen/StructLayout.java
index 7045f89..76cd351 100644
--- a/src/java/com/sun/gluegen/StructLayout.java
+++ b/src/java/com/sun/gluegen/StructLayout.java
@@ -120,7 +120,7 @@ public class StructLayout {
public static StructLayout createForCurrentPlatform() {
- // Note: this code is replicated in CPU.java
+ // Note: this code is replicated in (from?) Platform.java
String os = System.getProperty("os.name").toLowerCase();
String cpu = System.getProperty("os.arch").toLowerCase();
if ((os.startsWith("windows") && cpu.equals("x86"))) {
diff --git a/src/java/com/sun/gluegen/test/TestPointerBufferEndian.java b/src/java/com/sun/gluegen/test/TestPointerBufferEndian.java
index 0126b72..96e4e87 100644
--- a/src/java/com/sun/gluegen/test/TestPointerBufferEndian.java
+++ b/src/java/com/sun/gluegen/test/TestPointerBufferEndian.java
@@ -8,12 +8,12 @@ public class TestPointerBufferEndian {
public static void main (String[] args) {
boolean direct = args.length>0 && args[0].equals("-direct");
boolean ok = true;
- int bitsPtr = CPU.getPointerSizeInBits();
+ int bitsPtr = Platform.getPointerSizeInBits();
String bitsProp = System.getProperty("sun.arch.data.model");
String os = System.getProperty("os.name");
String cpu = System.getProperty("os.arch");
System.out.println("OS: <"+os+"> CPU: <"+cpu+"> Bits: <"+bitsPtr+"/"+bitsProp+">");
- System.out.println("CPU is: "+ (CPU.is32Bit()?"32":"64") + " bit");
+ System.out.println("CPU is: "+ (Platform.is32Bit()?"32":"64") + " bit");
System.out.println("Buffer is in: "+ (Platform.isLittleEndian()?"little":"big") + " endian");
PointerBuffer ptr = direct ? PointerBuffer.allocateDirect(3) : PointerBuffer.allocate(3);
ptr.put(0, 0x0123456789ABCDEFL);
diff --git a/src/java/com/sun/gluegen/test/TestStructAccessorEndian.java b/src/java/com/sun/gluegen/test/TestStructAccessorEndian.java
index 89ccb5a..7202056 100644
--- a/src/java/com/sun/gluegen/test/TestStructAccessorEndian.java
+++ b/src/java/com/sun/gluegen/test/TestStructAccessorEndian.java
@@ -7,12 +7,12 @@ import java.nio.*;
public class TestStructAccessorEndian {
public static void main (String args[]) {
boolean ok = true;
- int bitsPtr = CPU.getPointerSizeInBits();
+ int bitsPtr = Platform.getPointerSizeInBits();
String bitsProp = System.getProperty("sun.arch.data.model");
String os = System.getProperty("os.name");
String cpu = System.getProperty("os.arch");
System.out.println("OS: <"+os+"> CPU: <"+cpu+"> Bits: <"+bitsPtr+"/"+bitsProp+">");
- System.out.println("CPU is: "+ (CPU.is32Bit()?"32":"64") + " bit");
+ System.out.println("CPU is: "+ (Platform.is32Bit()?"32":"64") + " bit");
System.out.println("Buffer is in: "+ (Platform.isLittleEndian()?"little":"big") + " endian");
ByteBuffer tst = Buffers.newDirectByteBuffer(Buffers.SIZEOF_LONG * 3);
StructAccessor acc = new StructAccessor(tst);
diff --git a/src/java/com/sun/gluegen/JavaConfiguration.java b/src/java/com/sun/gluegen/JavaConfiguration.java
index 2bcb17a..e0a0eb6 100644
--- a/src/java/com/sun/gluegen/JavaConfiguration.java
+++ b/src/java/com/sun/gluegen/JavaConfiguration.java
@@ -104,7 +104,7 @@ public class JavaConfiguration {
/**
* The package in which the generated glue code expects to find its
* run-time helper classes (Buffer, CPU,
* StructAccessor). Defaults to "com.jogamp.gluegen.runtime".
*/
private String gluegenRuntimePackage = "com.jogamp.gluegen.runtime";
@@ -308,7 +308,7 @@ public class JavaConfiguration {
}
/** Returns the package in which the generated glue code expects to
- find its run-time helper classes (Buffer, CPU,
+ find its run-time helper classes (Buffers, Platform,
StructAccessor). Defaults to "com.jogamp.gluegen.runtime". */
public String gluegenRuntimePackage() {
return gluegenRuntimePackage;
@@ -104,7 +104,7 @@ public class JavaConfiguration {
/**
* The package in which the generated glue code expects to find its
- * run-time helper classes (Buffer, CPU,
+ * run-time helper classes (Buffers, Platform,
* StructAccessor). Defaults to "com.jogamp.gluegen.runtime".
*/